# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Type, Iterable, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.browser_rendering import content_create_params
from ...types.browser_rendering.content_create_response import ContentCreateResponse

__all__ = ["ContentResource", "AsyncContentResource"]


class ContentResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ContentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ContentResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        url: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[content_create_params.Variant0AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[content_create_params.Variant0AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: content_create_params.Variant0Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[content_create_params.Variant0Cookie] | Omit = omit,
        emulate_media_type: str | Omit = omit,
        goto_options: content_create_params.Variant0GotoOptions | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: content_create_params.Variant0Viewport | Omit = omit,
        wait_for_selector: content_create_params.Variant0WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Fetches rendered HTML content from provided URL or HTML.

        Check available options
        like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          url: URL to navigate to, eg. `https://example.com`.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        html: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[content_create_params.Variant1AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[content_create_params.Variant1AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: content_create_params.Variant1Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[content_create_params.Variant1Cookie] | Omit = omit,
        emulate_media_type: str | Omit = omit,
        goto_options: content_create_params.Variant1GotoOptions | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: content_create_params.Variant1Viewport | Omit = omit,
        wait_for_selector: content_create_params.Variant1WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Fetches rendered HTML content from provided URL or HTML.

        Check available options
        like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          html: Set the content of the page, eg: `<h1>Hello World!!</h1>`. Either `html` or
              `url` must be set.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "url"], ["account_id", "html"])
    def create(
        self,
        *,
        account_id: str,
        url: str | Omit = omit,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[content_create_params.Variant0AddScriptTag]
        | Iterable[content_create_params.Variant1AddScriptTag]
        | Omit = omit,
        add_style_tag: Iterable[content_create_params.Variant0AddStyleTag]
        | Iterable[content_create_params.Variant1AddStyleTag]
        | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: content_create_params.Variant0Authenticate
        | content_create_params.Variant1Authenticate
        | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[content_create_params.Variant0Cookie]
        | Iterable[content_create_params.Variant1Cookie]
        | Omit = omit,
        emulate_media_type: str | Omit = omit,
        goto_options: content_create_params.Variant0GotoOptions
        | content_create_params.Variant1GotoOptions
        | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: content_create_params.Variant0Viewport | content_create_params.Variant1Viewport | Omit = omit,
        wait_for_selector: content_create_params.Variant0WaitForSelector
        | content_create_params.Variant1WaitForSelector
        | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        html: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/browser-rendering/content", account_id=account_id),
            body=maybe_transform(
                {
                    "url": url,
                    "action_timeout": action_timeout,
                    "add_script_tag": add_script_tag,
                    "add_style_tag": add_style_tag,
                    "allow_request_pattern": allow_request_pattern,
                    "allow_resource_types": allow_resource_types,
                    "authenticate": authenticate,
                    "best_attempt": best_attempt,
                    "cookies": cookies,
                    "emulate_media_type": emulate_media_type,
                    "goto_options": goto_options,
                    "reject_request_pattern": reject_request_pattern,
                    "reject_resource_types": reject_resource_types,
                    "set_extra_http_headers": set_extra_http_headers,
                    "set_java_script_enabled": set_java_script_enabled,
                    "user_agent": user_agent,
                    "viewport": viewport,
                    "wait_for_selector": wait_for_selector,
                    "wait_for_timeout": wait_for_timeout,
                    "html": html,
                },
                content_create_params.ContentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"cache_ttl": cache_ttl}, content_create_params.ContentCreateParams),
                post_parser=ResultWrapper[Optional[ContentCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class AsyncContentResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncContentResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        url: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[content_create_params.Variant0AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[content_create_params.Variant0AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: content_create_params.Variant0Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[content_create_params.Variant0Cookie] | Omit = omit,
        emulate_media_type: str | Omit = omit,
        goto_options: content_create_params.Variant0GotoOptions | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: content_create_params.Variant0Viewport | Omit = omit,
        wait_for_selector: content_create_params.Variant0WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Fetches rendered HTML content from provided URL or HTML.

        Check available options
        like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          url: URL to navigate to, eg. `https://example.com`.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        html: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[content_create_params.Variant1AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[content_create_params.Variant1AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: content_create_params.Variant1Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[content_create_params.Variant1Cookie] | Omit = omit,
        emulate_media_type: str | Omit = omit,
        goto_options: content_create_params.Variant1GotoOptions | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: content_create_params.Variant1Viewport | Omit = omit,
        wait_for_selector: content_create_params.Variant1WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Fetches rendered HTML content from provided URL or HTML.

        Check available options
        like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          html: Set the content of the page, eg: `<h1>Hello World!!</h1>`. Either `html` or
              `url` must be set.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "url"], ["account_id", "html"])
    async def create(
        self,
        *,
        account_id: str,
        url: str | Omit = omit,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[content_create_params.Variant0AddScriptTag]
        | Iterable[content_create_params.Variant1AddScriptTag]
        | Omit = omit,
        add_style_tag: Iterable[content_create_params.Variant0AddStyleTag]
        | Iterable[content_create_params.Variant1AddStyleTag]
        | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: content_create_params.Variant0Authenticate
        | content_create_params.Variant1Authenticate
        | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[content_create_params.Variant0Cookie]
        | Iterable[content_create_params.Variant1Cookie]
        | Omit = omit,
        emulate_media_type: str | Omit = omit,
        goto_options: content_create_params.Variant0GotoOptions
        | content_create_params.Variant1GotoOptions
        | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        user_agent: str | Omit = omit,
        viewport: content_create_params.Variant0Viewport | content_create_params.Variant1Viewport | Omit = omit,
        wait_for_selector: content_create_params.Variant0WaitForSelector
        | content_create_params.Variant1WaitForSelector
        | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        html: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/browser-rendering/content", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "url": url,
                    "action_timeout": action_timeout,
                    "add_script_tag": add_script_tag,
                    "add_style_tag": add_style_tag,
                    "allow_request_pattern": allow_request_pattern,
                    "allow_resource_types": allow_resource_types,
                    "authenticate": authenticate,
                    "best_attempt": best_attempt,
                    "cookies": cookies,
                    "emulate_media_type": emulate_media_type,
                    "goto_options": goto_options,
                    "reject_request_pattern": reject_request_pattern,
                    "reject_resource_types": reject_resource_types,
                    "set_extra_http_headers": set_extra_http_headers,
                    "set_java_script_enabled": set_java_script_enabled,
                    "user_agent": user_agent,
                    "viewport": viewport,
                    "wait_for_selector": wait_for_selector,
                    "wait_for_timeout": wait_for_timeout,
                    "html": html,
                },
                content_create_params.ContentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"cache_ttl": cache_ttl}, content_create_params.ContentCreateParams),
                post_parser=ResultWrapper[Optional[ContentCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class ContentResourceWithRawResponse:
    def __init__(self, content: ContentResource) -> None:
        self._content = content

        self.create = to_raw_response_wrapper(
            content.create,
        )


class AsyncContentResourceWithRawResponse:
    def __init__(self, content: AsyncContentResource) -> None:
        self._content = content

        self.create = async_to_raw_response_wrapper(
            content.create,
        )


class ContentResourceWithStreamingResponse:
    def __init__(self, content: ContentResource) -> None:
        self._content = content

        self.create = to_streamed_response_wrapper(
            content.create,
        )


class AsyncContentResourceWithStreamingResponse:
    def __init__(self, content: AsyncContentResource) -> None:
        self._content = content

        self.create = async_to_streamed_response_wrapper(
            content.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .watermark import Watermark
from .allowed_origins import AllowedOrigins

__all__ = ["Video", "Input", "Playback", "PublicDetails", "Status"]


class Input(BaseModel):
    height: Optional[int] = None
    """The video height in pixels.

    A value of `-1` means the height is unknown. The value becomes available after
    the upload and before the video is ready.
    """

    width: Optional[int] = None
    """The video width in pixels.

    A value of `-1` means the width is unknown. The value becomes available after
    the upload and before the video is ready.
    """


class Playback(BaseModel):
    dash: Optional[str] = None
    """DASH Media Presentation Description for the video."""

    hls: Optional[str] = None
    """The HLS manifest for the video."""


class PublicDetails(BaseModel):
    """
    Public details for the video including title, share link, channel link, and logo.
    """

    channel_link: Optional[str] = None

    logo: Optional[str] = None

    media_id: Optional[int] = None

    share_link: Optional[str] = None

    title: Optional[str] = None


class Status(BaseModel):
    """Specifies a detailed status for a video.

    If the `state` is `inprogress` or `error`, the `step` field returns `encoding` or `manifest`. If the `state` is `inprogress`, `pctComplete` returns a number between 0 and 100 to indicate the approximate percent of completion. If the `state` is `error`, `errorReasonCode` and `errorReasonText` provide additional details.
    """

    error_reason_code: Optional[str] = FieldInfo(alias="errorReasonCode", default=None)
    """Specifies why the video failed to encode.

    This field is empty if the video is not in an `error` state. Preferred for
    programmatic use.
    """

    error_reason_text: Optional[str] = FieldInfo(alias="errorReasonText", default=None)
    """
    Specifies why the video failed to encode using a human readable error message in
    English. This field is empty if the video is not in an `error` state.
    """

    pct_complete: Optional[str] = FieldInfo(alias="pctComplete", default=None)
    """Indicates the progress as a percentage between 0 and 100."""

    state: Optional[
        Literal["pendingupload", "downloading", "queued", "inprogress", "ready", "error", "live-inprogress"]
    ] = None
    """Specifies the processing status for all quality levels for a video."""


class Video(BaseModel):
    allowed_origins: Optional[List[AllowedOrigins]] = FieldInfo(alias="allowedOrigins", default=None)
    """Lists the origins allowed to display the video.

    Enter allowed origin domains in an array and use `*` for wildcard subdomains.
    Empty arrays allow the video to be viewed on any origin.
    """

    clipped_from: Optional[str] = FieldInfo(alias="clippedFrom", default=None)
    """The unique identifier of the source video this video was clipped from."""

    created: Optional[datetime] = None
    """The date and time the media item was created."""

    creator: Optional[str] = None
    """A user-defined identifier for the media creator."""

    duration: Optional[float] = None
    """The duration of the video in seconds.

    A value of `-1` means the duration is unknown. The duration becomes available
    after the upload and before the video is ready.
    """

    input: Optional[Input] = None

    live_input: Optional[str] = FieldInfo(alias="liveInput", default=None)
    """The live input ID used to upload a video with Stream Live."""

    max_duration_seconds: Optional[int] = FieldInfo(alias="maxDurationSeconds", default=None)
    """The maximum duration in seconds for a video upload.

    Can be set for a video that is not yet uploaded to limit its duration. Uploads
    that exceed the specified duration will fail during processing. A value of `-1`
    means the value is unknown.
    """

    max_size_bytes: Optional[int] = FieldInfo(alias="maxSizeBytes", default=None)
    """The maximum size in bytes for the video upload."""

    meta: Optional[object] = None
    """
    A user modifiable key-value store used to reference other systems of record for
    managing videos.
    """

    modified: Optional[datetime] = None
    """The date and time the media item was last modified."""

    playback: Optional[Playback] = None

    preview: Optional[str] = None
    """The video's preview page URI. This field is omitted until encoding is complete."""

    public_details: Optional[PublicDetails] = FieldInfo(alias="publicDetails", default=None)
    """
    Public details for the video including title, share link, channel link, and
    logo.
    """

    ready_to_stream: Optional[bool] = FieldInfo(alias="readyToStream", default=None)
    """Indicates whether the video is playable.

    The field is empty if the video is not ready for viewing or the live stream is
    still in progress.
    """

    ready_to_stream_at: Optional[datetime] = FieldInfo(alias="readyToStreamAt", default=None)
    """Indicates the time at which the video became playable.

    The field is empty if the video is not ready for viewing or the live stream is
    still in progress.
    """

    require_signed_urls: Optional[bool] = FieldInfo(alias="requireSignedURLs", default=None)
    """Indicates whether the video can be a accessed using the UID.

    When set to `true`, a signed token must be generated with a signing key to view
    the video.
    """

    scheduled_deletion: Optional[datetime] = FieldInfo(alias="scheduledDeletion", default=None)
    """Indicates the date and time at which the video will be deleted.

    Omit the field to indicate no change, or include with a `null` value to remove
    an existing scheduled deletion. If specified, must be at least 30 days from
    upload time.
    """

    size: Optional[float] = None
    """The size of the media item in bytes."""

    status: Optional[Status] = None
    """Specifies a detailed status for a video.

    If the `state` is `inprogress` or `error`, the `step` field returns `encoding`
    or `manifest`. If the `state` is `inprogress`, `pctComplete` returns a number
    between 0 and 100 to indicate the approximate percent of completion. If the
    `state` is `error`, `errorReasonCode` and `errorReasonText` provide additional
    details.
    """

    thumbnail: Optional[str] = None
    """The media item's thumbnail URI.

    This field is omitted until encoding is complete.
    """

    thumbnail_timestamp_pct: Optional[float] = FieldInfo(alias="thumbnailTimestampPct", default=None)
    """
    The timestamp for a thumbnail image calculated as a percentage value of the
    video's duration. To convert from a second-wise timestamp to a percentage,
    divide the desired timestamp by the total duration of the video. If this value
    is not set, the default thumbnail image is taken from 0s of the video.
    """

    uid: Optional[str] = None
    """A Cloudflare-generated unique identifier for a media item."""

    uploaded: Optional[datetime] = None
    """The date and time the media item was uploaded."""

    upload_expiry: Optional[datetime] = FieldInfo(alias="uploadExpiry", default=None)
    """
    The date and time when the video upload URL is no longer valid for direct user
    uploads.
    """

    watermark: Optional[Watermark] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SiteInfoCreateParams"]


class SiteInfoCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    auto_install: bool
    """
    If enabled, the JavaScript snippet is automatically injected for orange-clouded
    sites.
    """

    host: str
    """The hostname to use for gray-clouded sites."""

    zone_tag: str
    """The zone identifier."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DOHUpdateParams"]


class DOHUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    doh_jwt_duration: str
    """The duration the DoH JWT is valid for.

    Must be in the format `300ms` or `2h45m`. Valid time units are: ns, us (or µs),
    ms, s, m, h. Note that the maximum duration for this setting is the same as the
    key rotation period on the account. Default expiration is 24h
    """

    service_token_id: str
    """The uuid of the service token you want to use for DoH authentication"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ServiceTokenRuleParam", "ServiceToken"]


class ServiceToken(TypedDict, total=False):
    token_id: Required[str]
    """The ID of a Service Token."""


class ServiceTokenRuleParam(TypedDict, total=False):
    """Matches a specific Access Service Token"""

    service_token: Required[ServiceToken]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .email import (
    EmailResource,
    AsyncEmailResource,
    EmailResourceWithRawResponse,
    AsyncEmailResourceWithRawResponse,
    EmailResourceWithStreamingResponse,
    AsyncEmailResourceWithStreamingResponse,
)
from .routing import (
    RoutingResource,
    AsyncRoutingResource,
    RoutingResourceWithRawResponse,
    AsyncRoutingResourceWithRawResponse,
    RoutingResourceWithStreamingResponse,
    AsyncRoutingResourceWithStreamingResponse,
)
from .security import (
    SecurityResource,
    AsyncSecurityResource,
    SecurityResourceWithRawResponse,
    AsyncSecurityResourceWithRawResponse,
    SecurityResourceWithStreamingResponse,
    AsyncSecurityResourceWithStreamingResponse,
)

__all__ = [
    "RoutingResource",
    "AsyncRoutingResource",
    "RoutingResourceWithRawResponse",
    "AsyncRoutingResourceWithRawResponse",
    "RoutingResourceWithStreamingResponse",
    "AsyncRoutingResourceWithStreamingResponse",
    "SecurityResource",
    "AsyncSecurityResource",
    "SecurityResourceWithRawResponse",
    "AsyncSecurityResourceWithRawResponse",
    "SecurityResourceWithStreamingResponse",
    "AsyncSecurityResourceWithStreamingResponse",
    "EmailResource",
    "AsyncEmailResource",
    "EmailResourceWithRawResponse",
    "AsyncEmailResourceWithRawResponse",
    "EmailResourceWithStreamingResponse",
    "AsyncEmailResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["IPV6Configuration"]


class IPV6Configuration(BaseModel):
    target: Optional[Literal["ip6"]] = None
    """The configuration target.

    You must set the target to `ip6` when specifying an IPv6 address in the rule.
    """

    value: Optional[str] = None
    """The IPv6 address to match."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ZoneAuthenticatedOriginPull"]


class ZoneAuthenticatedOriginPull(BaseModel):
    id: Optional[str] = None
    """Identifier."""

    certificate: Optional[str] = None
    """The zone's leaf certificate."""

    expires_on: Optional[datetime] = None
    """When the certificate from the authority expires."""

    issuer: Optional[str] = None
    """The certificate authority that issued the certificate."""

    signature: Optional[str] = None
    """The type of hash used for the certificate."""

    status: Optional[
        Literal[
            "initializing",
            "pending_deployment",
            "pending_deletion",
            "active",
            "deleted",
            "deployment_timed_out",
            "deletion_timed_out",
        ]
    ] = None
    """Status of the certificate activation."""

    uploaded_on: Optional[datetime] = None
    """This is the time the certificate was uploaded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["ScopeParam"]


class ScopeParam(TypedDict, total=False):
    """Used only for ECMP routes."""

    colo_names: SequenceNotStr[str]
    """List of colo names for the ECMP scope."""

    colo_regions: SequenceNotStr[str]
    """List of colo regions for the ECMP scope."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .deployment import Deployment

__all__ = ["DeploymentListResponse"]


class DeploymentListResponse(BaseModel):
    deployments: List[Deployment]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["LivestreamGetAllLivestreamsParams"]


class LivestreamGetAllLivestreamsParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    end_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Specify the end time range in ISO format to access the live stream."""

    exclude_meetings: bool
    """Exclude the RealtimeKit meetings that are livestreamed."""

    page_no: int
    """The page number from which you want your page search results to be displayed."""

    per_page: int
    """Number of results per page."""

    sort_order: Literal["ASC", "DSC"]
    """Specifies the sorting order for the results."""

    start_time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Specify the start time range in ISO format to access the live stream."""

    status: Literal["LIVE", "IDLE", "ERRORED", "INVOKED"]
    """Specifies the status of the operation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tables import (
    TablesResource,
    AsyncTablesResource,
    TablesResourceWithRawResponse,
    AsyncTablesResourceWithRawResponse,
    TablesResourceWithStreamingResponse,
    AsyncTablesResourceWithStreamingResponse,
)
from .namespaces import (
    NamespacesResource,
    AsyncNamespacesResource,
    NamespacesResourceWithRawResponse,
    AsyncNamespacesResourceWithRawResponse,
    NamespacesResourceWithStreamingResponse,
    AsyncNamespacesResourceWithStreamingResponse,
)

__all__ = [
    "TablesResource",
    "AsyncTablesResource",
    "TablesResourceWithRawResponse",
    "AsyncTablesResourceWithRawResponse",
    "TablesResourceWithStreamingResponse",
    "AsyncTablesResourceWithStreamingResponse",
    "NamespacesResource",
    "AsyncNamespacesResource",
    "NamespacesResourceWithRawResponse",
    "AsyncNamespacesResourceWithRawResponse",
    "NamespacesResourceWithStreamingResponse",
    "AsyncNamespacesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OriginCloudRegion"]


class OriginCloudRegion(BaseModel):
    """A single origin IP-to-cloud-region mapping."""

    origin_ip: str
    """The origin IP address (IPv4 or IPv6).

    Normalized to canonical form (RFC 5952 for IPv6).
    """

    region: str
    """Cloud vendor region identifier."""

    vendor: Literal["aws", "azure", "gcp", "oci"]
    """Cloud vendor hosting the origin."""

    modified_on: Optional[datetime] = None
    """Time this mapping was last modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TargetParam", "Constraint"]


class Constraint(TypedDict, total=False):
    """String constraint."""

    operator: Required[Literal["matches", "contains", "equals", "not_equal", "not_contain"]]
    """
    The matches operator can use asterisks and pipes as wildcard and 'or' operators.
    """

    value: Required[str]
    """The URL pattern to match against the current request.

    The pattern may contain up to four asterisks ('\\**') as placeholders.
    """


class TargetParam(TypedDict, total=False):
    """URL target."""

    constraint: Constraint
    """String constraint."""

    target: Literal["url"]
    """A target based on the URL of the request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["MessagePushParams", "MqQueueMessageText", "MqQueueMessageJson"]


class MqQueueMessageText(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    body: str

    content_type: Literal["text"]

    delay_seconds: float
    """
    The number of seconds to wait for attempting to deliver this message to
    consumers
    """


class MqQueueMessageJson(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    body: object

    content_type: Literal["json"]

    delay_seconds: float
    """
    The number of seconds to wait for attempting to deliver this message to
    consumers
    """


MessagePushParams: TypeAlias = Union[MqQueueMessageText, MqQueueMessageJson]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .status import Status as Status
from .account import Account as Account
from .role_list_params import RoleListParams as RoleListParams
from .token_list_params import TokenListParams as TokenListParams
from .member_list_params import MemberListParams as MemberListParams
from .account_list_params import AccountListParams as AccountListParams
from .token_create_params import TokenCreateParams as TokenCreateParams
from .token_update_params import TokenUpdateParams as TokenUpdateParams
from .member_create_params import MemberCreateParams as MemberCreateParams
from .member_update_params import MemberUpdateParams as MemberUpdateParams
from .account_create_params import AccountCreateParams as AccountCreateParams
from .account_update_params import AccountUpdateParams as AccountUpdateParams
from .token_create_response import TokenCreateResponse as TokenCreateResponse
from .token_delete_response import TokenDeleteResponse as TokenDeleteResponse
from .token_verify_response import TokenVerifyResponse as TokenVerifyResponse
from .member_delete_response import MemberDeleteResponse as MemberDeleteResponse
from .account_delete_response import AccountDeleteResponse as AccountDeleteResponse
from .subscription_create_params import SubscriptionCreateParams as SubscriptionCreateParams
from .subscription_update_params import SubscriptionUpdateParams as SubscriptionUpdateParams
from .subscription_delete_response import SubscriptionDeleteResponse as SubscriptionDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import FileTypes

__all__ = ["LogoCreateParams"]


class LogoCreateParams(TypedDict, total=False):
    account_id: Required[str]

    match_type: str

    tag: str

    threshold: float

    image: FileTypes



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .items import (
    ItemsResource,
    AsyncItemsResource,
    ItemsResourceWithRawResponse,
    AsyncItemsResourceWithRawResponse,
    ItemsResourceWithStreamingResponse,
    AsyncItemsResourceWithStreamingResponse,
)
from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ......_base_client import AsyncPaginator, make_request_options
from ......types.ddos_protection.advanced_tcp_protection.syn_protection import filter_list_params, filter_create_params
from ......types.ddos_protection.advanced_tcp_protection.syn_protection.filter_list_response import FilterListResponse
from ......types.ddos_protection.advanced_tcp_protection.syn_protection.filter_create_response import (
    FilterCreateResponse,
)
from ......types.ddos_protection.advanced_tcp_protection.syn_protection.filter_bulk_delete_response import (
    FilterBulkDeleteResponse,
)

__all__ = ["FiltersResource", "AsyncFiltersResource"]


class FiltersResource(SyncAPIResource):
    @cached_property
    def items(self) -> ItemsResource:
        return ItemsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FiltersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FiltersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FiltersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FiltersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        expression: str,
        mode: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FilterCreateResponse]:
        """
        Create a SYN Protection filter for an account.

        Args:
          account_id: Identifier.

          expression: The filter expression.

          mode: The filter's mode. Must be one of 'enabled', 'disabled', 'monitoring'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters",
                account_id=account_id,
            ),
            body=maybe_transform(
                {
                    "expression": expression,
                    "mode": mode,
                },
                filter_create_params.FilterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FilterCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FilterCreateResponse]], ResultWrapper[FilterCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: str | Omit = omit,
        mode: str | Omit = omit,
        order: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[FilterListResponse]:
        """
        List all SYN Protection filters for an account.

        Args:
          account_id: Identifier.

          direction: The direction of ordering (ASC or DESC). Defaults to 'ASC'.

          mode: The mode of the filters to get. Optional. Valid values: 'enabled', 'disabled',
              'monitoring'.

          order: The field to order by. Defaults to 'prefix'.

          page: The page number for pagination. Defaults to 1.

          per_page: The number of items per page. Must be between 10 and 1000. Defaults to 25.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters",
                account_id=account_id,
            ),
            page=SyncV4PagePaginationArray[FilterListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "mode": mode,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    filter_list_params.FilterListParams,
                ),
            ),
            model=FilterListResponse,
        )

    def bulk_delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FilterBulkDeleteResponse:
        """
        Delete all SYN Protection filters for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters",
                account_id=account_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FilterBulkDeleteResponse,
        )


class AsyncFiltersResource(AsyncAPIResource):
    @cached_property
    def items(self) -> AsyncItemsResource:
        return AsyncItemsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFiltersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFiltersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFiltersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFiltersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        expression: str,
        mode: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[FilterCreateResponse]:
        """
        Create a SYN Protection filter for an account.

        Args:
          account_id: Identifier.

          expression: The filter expression.

          mode: The filter's mode. Must be one of 'enabled', 'disabled', 'monitoring'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters",
                account_id=account_id,
            ),
            body=await async_maybe_transform(
                {
                    "expression": expression,
                    "mode": mode,
                },
                filter_create_params.FilterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[FilterCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[FilterCreateResponse]], ResultWrapper[FilterCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: str | Omit = omit,
        mode: str | Omit = omit,
        order: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FilterListResponse, AsyncV4PagePaginationArray[FilterListResponse]]:
        """
        List all SYN Protection filters for an account.

        Args:
          account_id: Identifier.

          direction: The direction of ordering (ASC or DESC). Defaults to 'ASC'.

          mode: The mode of the filters to get. Optional. Valid values: 'enabled', 'disabled',
              'monitoring'.

          order: The field to order by. Defaults to 'prefix'.

          page: The page number for pagination. Defaults to 1.

          per_page: The number of items per page. Must be between 10 and 1000. Defaults to 25.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters",
                account_id=account_id,
            ),
            page=AsyncV4PagePaginationArray[FilterListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "mode": mode,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    filter_list_params.FilterListParams,
                ),
            ),
            model=FilterListResponse,
        )

    async def bulk_delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FilterBulkDeleteResponse:
        """
        Delete all SYN Protection filters for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/advanced_tcp_protection/configs/syn_protection/filters",
                account_id=account_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FilterBulkDeleteResponse,
        )


class FiltersResourceWithRawResponse:
    def __init__(self, filters: FiltersResource) -> None:
        self._filters = filters

        self.create = to_raw_response_wrapper(
            filters.create,
        )
        self.list = to_raw_response_wrapper(
            filters.list,
        )
        self.bulk_delete = to_raw_response_wrapper(
            filters.bulk_delete,
        )

    @cached_property
    def items(self) -> ItemsResourceWithRawResponse:
        return ItemsResourceWithRawResponse(self._filters.items)


class AsyncFiltersResourceWithRawResponse:
    def __init__(self, filters: AsyncFiltersResource) -> None:
        self._filters = filters

        self.create = async_to_raw_response_wrapper(
            filters.create,
        )
        self.list = async_to_raw_response_wrapper(
            filters.list,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            filters.bulk_delete,
        )

    @cached_property
    def items(self) -> AsyncItemsResourceWithRawResponse:
        return AsyncItemsResourceWithRawResponse(self._filters.items)


class FiltersResourceWithStreamingResponse:
    def __init__(self, filters: FiltersResource) -> None:
        self._filters = filters

        self.create = to_streamed_response_wrapper(
            filters.create,
        )
        self.list = to_streamed_response_wrapper(
            filters.list,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            filters.bulk_delete,
        )

    @cached_property
    def items(self) -> ItemsResourceWithStreamingResponse:
        return ItemsResourceWithStreamingResponse(self._filters.items)


class AsyncFiltersResourceWithStreamingResponse:
    def __init__(self, filters: AsyncFiltersResource) -> None:
        self._filters = filters

        self.create = async_to_streamed_response_wrapper(
            filters.create,
        )
        self.list = async_to_streamed_response_wrapper(
            filters.list,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            filters.bulk_delete,
        )

    @cached_property
    def items(self) -> AsyncItemsResourceWithStreamingResponse:
        return AsyncItemsResourceWithStreamingResponse(self._filters.items)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["RouteStatsParams"]


class RouteStatsParams(TypedDict, total=False):
    asn: int
    """Filters results by Autonomous System.

    Specify a single Autonomous System Number (ASN) as integer.
    """

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    location: str
    """Filters results by location. Specify an alpha-2 location code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .members import (
    MembersResource,
    AsyncMembersResource,
    MembersResourceWithRawResponse,
    AsyncMembersResourceWithRawResponse,
    MembersResourceWithStreamingResponse,
    AsyncMembersResourceWithStreamingResponse,
)
from .user_groups import (
    UserGroupsResource,
    AsyncUserGroupsResource,
    UserGroupsResourceWithRawResponse,
    AsyncUserGroupsResourceWithRawResponse,
    UserGroupsResourceWithStreamingResponse,
    AsyncUserGroupsResourceWithStreamingResponse,
)

__all__ = [
    "MembersResource",
    "AsyncMembersResource",
    "MembersResourceWithRawResponse",
    "AsyncMembersResourceWithRawResponse",
    "MembersResourceWithStreamingResponse",
    "AsyncMembersResourceWithStreamingResponse",
    "UserGroupsResource",
    "AsyncUserGroupsResource",
    "UserGroupsResourceWithRawResponse",
    "AsyncUserGroupsResourceWithRawResponse",
    "UserGroupsResourceWithStreamingResponse",
    "AsyncUserGroupsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cmb import (
    CmbResource,
    AsyncCmbResource,
    CmbResourceWithRawResponse,
    AsyncCmbResourceWithRawResponse,
    CmbResourceWithStreamingResponse,
    AsyncCmbResourceWithStreamingResponse,
)
from .control import (
    ControlResource,
    AsyncControlResource,
    ControlResourceWithRawResponse,
    AsyncControlResourceWithRawResponse,
    ControlResourceWithStreamingResponse,
    AsyncControlResourceWithStreamingResponse,
)
from .retention import (
    RetentionResource,
    AsyncRetentionResource,
    RetentionResourceWithRawResponse,
    AsyncRetentionResourceWithRawResponse,
    RetentionResourceWithStreamingResponse,
    AsyncRetentionResourceWithStreamingResponse,
)

__all__ = [
    "RetentionResource",
    "AsyncRetentionResource",
    "RetentionResourceWithRawResponse",
    "AsyncRetentionResourceWithRawResponse",
    "RetentionResourceWithStreamingResponse",
    "AsyncRetentionResourceWithStreamingResponse",
    "CmbResource",
    "AsyncCmbResource",
    "CmbResourceWithRawResponse",
    "AsyncCmbResourceWithRawResponse",
    "CmbResourceWithStreamingResponse",
    "AsyncCmbResourceWithStreamingResponse",
    "ControlResource",
    "AsyncControlResource",
    "ControlResourceWithRawResponse",
    "AsyncControlResourceWithRawResponse",
    "ControlResourceWithStreamingResponse",
    "AsyncControlResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ScanScreenshotParams"]


class ScanScreenshotParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    resolution: Literal["desktop", "mobile", "tablet"]
    """Target device type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RecordImportParams"]


class RecordImportParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    file: Required[str]
    """BIND config to import.

    **Tip:** When using cURL, a file can be uploaded using
    `--form 'file=@bind_config.txt'`.
    """

    proxied: str
    """
    Whether or not proxiable records should receive the performance and security
    benefits of Cloudflare.

    The value should be either `true` or `false`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .quota import (
    QuotaResource,
    AsyncQuotaResource,
    QuotaResourceWithRawResponse,
    AsyncQuotaResourceWithRawResponse,
    QuotaResourceWithStreamingResponse,
    AsyncQuotaResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .stores.stores import (
    StoresResource,
    AsyncStoresResource,
    StoresResourceWithRawResponse,
    AsyncStoresResourceWithRawResponse,
    StoresResourceWithStreamingResponse,
    AsyncStoresResourceWithStreamingResponse,
)

__all__ = ["SecretsStoreResource", "AsyncSecretsStoreResource"]


class SecretsStoreResource(SyncAPIResource):
    @cached_property
    def stores(self) -> StoresResource:
        return StoresResource(self._client)

    @cached_property
    def quota(self) -> QuotaResource:
        return QuotaResource(self._client)

    @cached_property
    def with_raw_response(self) -> SecretsStoreResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SecretsStoreResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SecretsStoreResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SecretsStoreResourceWithStreamingResponse(self)


class AsyncSecretsStoreResource(AsyncAPIResource):
    @cached_property
    def stores(self) -> AsyncStoresResource:
        return AsyncStoresResource(self._client)

    @cached_property
    def quota(self) -> AsyncQuotaResource:
        return AsyncQuotaResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSecretsStoreResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSecretsStoreResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSecretsStoreResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSecretsStoreResourceWithStreamingResponse(self)


class SecretsStoreResourceWithRawResponse:
    def __init__(self, secrets_store: SecretsStoreResource) -> None:
        self._secrets_store = secrets_store

    @cached_property
    def stores(self) -> StoresResourceWithRawResponse:
        return StoresResourceWithRawResponse(self._secrets_store.stores)

    @cached_property
    def quota(self) -> QuotaResourceWithRawResponse:
        return QuotaResourceWithRawResponse(self._secrets_store.quota)


class AsyncSecretsStoreResourceWithRawResponse:
    def __init__(self, secrets_store: AsyncSecretsStoreResource) -> None:
        self._secrets_store = secrets_store

    @cached_property
    def stores(self) -> AsyncStoresResourceWithRawResponse:
        return AsyncStoresResourceWithRawResponse(self._secrets_store.stores)

    @cached_property
    def quota(self) -> AsyncQuotaResourceWithRawResponse:
        return AsyncQuotaResourceWithRawResponse(self._secrets_store.quota)


class SecretsStoreResourceWithStreamingResponse:
    def __init__(self, secrets_store: SecretsStoreResource) -> None:
        self._secrets_store = secrets_store

    @cached_property
    def stores(self) -> StoresResourceWithStreamingResponse:
        return StoresResourceWithStreamingResponse(self._secrets_store.stores)

    @cached_property
    def quota(self) -> QuotaResourceWithStreamingResponse:
        return QuotaResourceWithStreamingResponse(self._secrets_store.quota)


class AsyncSecretsStoreResourceWithStreamingResponse:
    def __init__(self, secrets_store: AsyncSecretsStoreResource) -> None:
        self._secrets_store = secrets_store

    @cached_property
    def stores(self) -> AsyncStoresResourceWithStreamingResponse:
        return AsyncStoresResourceWithStreamingResponse(self._secrets_store.stores)

    @cached_property
    def quota(self) -> AsyncQuotaResourceWithStreamingResponse:
        return AsyncQuotaResourceWithStreamingResponse(self._secrets_store.quota)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .scans import (
    ScansResource,
    AsyncScansResource,
    ScansResourceWithRawResponse,
    AsyncScansResourceWithRawResponse,
    ScansResourceWithStreamingResponse,
    AsyncScansResourceWithStreamingResponse,
)
from .credential_sets import (
    CredentialSetsResource,
    AsyncCredentialSetsResource,
    CredentialSetsResourceWithRawResponse,
    AsyncCredentialSetsResourceWithRawResponse,
    CredentialSetsResourceWithStreamingResponse,
    AsyncCredentialSetsResourceWithStreamingResponse,
)
from .target_environments import (
    TargetEnvironmentsResource,
    AsyncTargetEnvironmentsResource,
    TargetEnvironmentsResourceWithRawResponse,
    AsyncTargetEnvironmentsResourceWithRawResponse,
    TargetEnvironmentsResourceWithStreamingResponse,
    AsyncTargetEnvironmentsResourceWithStreamingResponse,
)
from .vulnerability_scanner import (
    VulnerabilityScannerResource,
    AsyncVulnerabilityScannerResource,
    VulnerabilityScannerResourceWithRawResponse,
    AsyncVulnerabilityScannerResourceWithRawResponse,
    VulnerabilityScannerResourceWithStreamingResponse,
    AsyncVulnerabilityScannerResourceWithStreamingResponse,
)

__all__ = [
    "CredentialSetsResource",
    "AsyncCredentialSetsResource",
    "CredentialSetsResourceWithRawResponse",
    "AsyncCredentialSetsResourceWithRawResponse",
    "CredentialSetsResourceWithStreamingResponse",
    "AsyncCredentialSetsResourceWithStreamingResponse",
    "ScansResource",
    "AsyncScansResource",
    "ScansResourceWithRawResponse",
    "AsyncScansResourceWithRawResponse",
    "ScansResourceWithStreamingResponse",
    "AsyncScansResourceWithStreamingResponse",
    "TargetEnvironmentsResource",
    "AsyncTargetEnvironmentsResource",
    "TargetEnvironmentsResourceWithRawResponse",
    "AsyncTargetEnvironmentsResourceWithRawResponse",
    "TargetEnvironmentsResourceWithStreamingResponse",
    "AsyncTargetEnvironmentsResourceWithStreamingResponse",
    "VulnerabilityScannerResource",
    "AsyncVulnerabilityScannerResource",
    "VulnerabilityScannerResourceWithRawResponse",
    "AsyncVulnerabilityScannerResourceWithRawResponse",
    "VulnerabilityScannerResourceWithStreamingResponse",
    "AsyncVulnerabilityScannerResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["VideoStorageUsageResponse"]


class VideoStorageUsageResponse(BaseModel):
    creator: Optional[str] = None
    """A user-defined identifier for the media creator."""

    total_storage_minutes: Optional[float] = FieldInfo(alias="totalStorageMinutes", default=None)
    """The total minutes of video content stored in the account.

    May contain decimal values.
    """

    total_storage_minutes_limit: Optional[int] = FieldInfo(alias="totalStorageMinutesLimit", default=None)
    """The storage capacity alloted for the account."""

    video_count: Optional[int] = FieldInfo(alias="videoCount", default=None)
    """The total count of videos associated with the account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .item_edit_params import ItemEditParams as ItemEditParams
from .item_get_response import ItemGetResponse as ItemGetResponse
from .item_edit_response import ItemEditResponse as ItemEditResponse
from .item_delete_response import ItemDeleteResponse as ItemDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MitigationListParams"]


class MitigationListParams(TypedDict, total=False):
    account_id: Required[str]

    effective_after: str
    """Returns mitigation that were dispatched after the given date"""

    effective_before: str
    """Returns mitigations that were dispatched before the given date"""

    entity_type: Literal["url_pattern", "account", "zone"]
    """Filter by the type of entity the mitigation impacts."""

    page: int
    """Where in pagination to start listing abuse reports"""

    per_page: int
    """How many abuse reports per page to list"""

    sort: Literal[
        "type,asc",
        "type,desc",
        "effective_date,asc",
        "effective_date,desc",
        "status,asc",
        "status,desc",
        "entity_type,asc",
        "entity_type,desc",
    ]
    """A property to sort by, followed by the order"""

    status: Literal["pending", "active", "in_review", "cancelled", "removed"]
    """Filter by the status of the mitigation."""

    type: Literal[
        "legal_block",
        "misleading_interstitial",
        "phishing_interstitial",
        "network_block",
        "rate_limit_cache",
        "account_suspend",
        "redirect_video_stream",
    ]
    """Filter by the type of mitigation.

    This filter parameter can be specified multiple times to include multiple types
    of mitigations in the result set, e.g. ?type=rate_limit_cache&type=legal_block.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .subnet import Subnet
from ...._types import SequenceNotStr

__all__ = ["ACLConfigurationParam"]


class ACLConfigurationParam(TypedDict, total=False):
    lan_id: Required[str]
    """The identifier for the LAN you want to create an ACL policy with."""

    lan_name: str
    """The name of the LAN based on the provided lan_id."""

    port_ranges: SequenceNotStr[str]
    """Array of port ranges on the provided LAN that will be included in the ACL.

    If no ports or port rangess are provided, communication on any port on this LAN
    is allowed.
    """

    ports: Iterable[int]
    """Array of ports on the provided LAN that will be included in the ACL.

    If no ports or port ranges are provided, communication on any port on this LAN
    is allowed.
    """

    subnets: SequenceNotStr[Subnet]
    """Array of subnet IPs within the LAN that will be included in the ACL.

    If no subnets are provided, communication on any subnets on this LAN are
    allowed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.ssl import analyze_create_params
from ..._base_client import make_request_options
from ...types.custom_hostnames import BundleMethod
from ...types.custom_hostnames.bundle_method import BundleMethod

__all__ = ["AnalyzeResource", "AsyncAnalyzeResource"]


class AnalyzeResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AnalyzeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AnalyzeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AnalyzeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AnalyzeResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        bundle_method: BundleMethod | Omit = omit,
        certificate: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Returns the set of hostnames, the signature algorithm, and the expiration date
        of the certificate.

        Args:
          zone_id: Identifier.

          bundle_method: A ubiquitous bundle has the highest probability of being verified everywhere,
              even by clients using outdated or unusual trust stores. An optimal bundle uses
              the shortest chain and newest intermediates. And the force bundle verifies the
              chain, but does not otherwise modify it.

          certificate: The zone's SSL certificate or certificate and the intermediate(s).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/ssl/analyze", zone_id=zone_id),
            body=maybe_transform(
                {
                    "bundle_method": bundle_method,
                    "certificate": certificate,
                },
                analyze_create_params.AnalyzeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncAnalyzeResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAnalyzeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAnalyzeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAnalyzeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAnalyzeResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        bundle_method: BundleMethod | Omit = omit,
        certificate: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Returns the set of hostnames, the signature algorithm, and the expiration date
        of the certificate.

        Args:
          zone_id: Identifier.

          bundle_method: A ubiquitous bundle has the highest probability of being verified everywhere,
              even by clients using outdated or unusual trust stores. An optimal bundle uses
              the shortest chain and newest intermediates. And the force bundle verifies the
              chain, but does not otherwise modify it.

          certificate: The zone's SSL certificate or certificate and the intermediate(s).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/ssl/analyze", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "bundle_method": bundle_method,
                    "certificate": certificate,
                },
                analyze_create_params.AnalyzeCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AnalyzeResourceWithRawResponse:
    def __init__(self, analyze: AnalyzeResource) -> None:
        self._analyze = analyze

        self.create = to_raw_response_wrapper(
            analyze.create,
        )


class AsyncAnalyzeResourceWithRawResponse:
    def __init__(self, analyze: AsyncAnalyzeResource) -> None:
        self._analyze = analyze

        self.create = async_to_raw_response_wrapper(
            analyze.create,
        )


class AnalyzeResourceWithStreamingResponse:
    def __init__(self, analyze: AnalyzeResource) -> None:
        self._analyze = analyze

        self.create = to_streamed_response_wrapper(
            analyze.create,
        )


class AsyncAnalyzeResourceWithStreamingResponse:
    def __init__(self, analyze: AsyncAnalyzeResource) -> None:
        self._analyze = analyze

        self.create = async_to_streamed_response_wrapper(
            analyze.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tls import (
    TLSResource,
    AsyncTLSResource,
    TLSResourceWithRawResponse,
    AsyncTLSResourceWithRawResponse,
    TLSResourceWithStreamingResponse,
    AsyncTLSResourceWithStreamingResponse,
)
from .origin import (
    OriginResource,
    AsyncOriginResource,
    OriginResourceWithRawResponse,
    AsyncOriginResourceWithRawResponse,
    OriginResourceWithStreamingResponse,
    AsyncOriginResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["PostQuantumResource", "AsyncPostQuantumResource"]


class PostQuantumResource(SyncAPIResource):
    @cached_property
    def origin(self) -> OriginResource:
        return OriginResource(self._client)

    @cached_property
    def tls(self) -> TLSResource:
        return TLSResource(self._client)

    @cached_property
    def with_raw_response(self) -> PostQuantumResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PostQuantumResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PostQuantumResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PostQuantumResourceWithStreamingResponse(self)


class AsyncPostQuantumResource(AsyncAPIResource):
    @cached_property
    def origin(self) -> AsyncOriginResource:
        return AsyncOriginResource(self._client)

    @cached_property
    def tls(self) -> AsyncTLSResource:
        return AsyncTLSResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPostQuantumResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPostQuantumResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPostQuantumResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPostQuantumResourceWithStreamingResponse(self)


class PostQuantumResourceWithRawResponse:
    def __init__(self, post_quantum: PostQuantumResource) -> None:
        self._post_quantum = post_quantum

    @cached_property
    def origin(self) -> OriginResourceWithRawResponse:
        return OriginResourceWithRawResponse(self._post_quantum.origin)

    @cached_property
    def tls(self) -> TLSResourceWithRawResponse:
        return TLSResourceWithRawResponse(self._post_quantum.tls)


class AsyncPostQuantumResourceWithRawResponse:
    def __init__(self, post_quantum: AsyncPostQuantumResource) -> None:
        self._post_quantum = post_quantum

    @cached_property
    def origin(self) -> AsyncOriginResourceWithRawResponse:
        return AsyncOriginResourceWithRawResponse(self._post_quantum.origin)

    @cached_property
    def tls(self) -> AsyncTLSResourceWithRawResponse:
        return AsyncTLSResourceWithRawResponse(self._post_quantum.tls)


class PostQuantumResourceWithStreamingResponse:
    def __init__(self, post_quantum: PostQuantumResource) -> None:
        self._post_quantum = post_quantum

    @cached_property
    def origin(self) -> OriginResourceWithStreamingResponse:
        return OriginResourceWithStreamingResponse(self._post_quantum.origin)

    @cached_property
    def tls(self) -> TLSResourceWithStreamingResponse:
        return TLSResourceWithStreamingResponse(self._post_quantum.tls)


class AsyncPostQuantumResourceWithStreamingResponse:
    def __init__(self, post_quantum: AsyncPostQuantumResource) -> None:
        self._post_quantum = post_quantum

    @cached_property
    def origin(self) -> AsyncOriginResourceWithStreamingResponse:
        return AsyncOriginResourceWithStreamingResponse(self._post_quantum.origin)

    @cached_property
    def tls(self) -> AsyncTLSResourceWithStreamingResponse:
        return AsyncTLSResourceWithStreamingResponse(self._post_quantum.tls)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["EventListParams"]


class EventListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier"""

    from_: Required[Annotated[float, PropertyInfo(alias="from")]]

    to: Required[float]

    cursor: str

    k: str
    """Filter by event kind"""

    limit: float



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RecordingGetActiveRecordingsResponse", "Data"]


class Data(BaseModel):
    """Data returned by the operation"""

    id: str
    """ID of the recording"""

    audio_download_url: Optional[str] = None
    """
    If the audio_config is passed, the URL for downloading the audio recording is
    returned.
    """

    download_url: Optional[str] = None
    """URL where the recording can be downloaded."""

    download_url_expiry: Optional[datetime] = None
    """Timestamp when the download URL expires."""

    file_size: Optional[float] = None
    """File size of the recording, in bytes."""

    invoked_time: datetime
    """Timestamp when this recording was invoked."""

    output_file_name: str
    """File name of the recording."""

    session_id: Optional[str] = None
    """ID of the meeting session this recording is for."""

    started_time: Optional[datetime] = None
    """Timestamp when this recording actually started after being invoked.

    Usually a few seconds after `invoked_time`.
    """

    status: Literal["INVOKED", "RECORDING", "UPLOADING", "UPLOADED", "ERRORED", "PAUSED"]
    """Current status of the recording."""

    stopped_time: Optional[datetime] = None
    """Timestamp when this recording was stopped.

    Optional; is present only when the recording has actually been stopped.
    """

    recording_duration: Optional[int] = None
    """Total recording time in seconds."""


class RecordingGetActiveRecordingsResponse(BaseModel):
    data: Data
    """Data returned by the operation"""

    success: bool
    """Success status of the operation"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from .details import (
    DetailsResource,
    AsyncDetailsResource,
    DetailsResourceWithRawResponse,
    AsyncDetailsResourceWithRawResponse,
    DetailsResourceWithStreamingResponse,
    AsyncDetailsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.waiting_rooms import event_edit_params, event_list_params, event_create_params, event_update_params
from ....types.waiting_rooms.event import Event
from ....types.waiting_rooms.event_delete_response import EventDeleteResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def details(self) -> DetailsResource:
        return DetailsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def create(
        self,
        waiting_room_id: str,
        *,
        zone_id: str,
        event_end_time: str,
        event_start_time: str,
        name: str,
        custom_page_html: Optional[str] | Omit = omit,
        description: str | Omit = omit,
        disable_session_renewal: Optional[bool] | Omit = omit,
        new_users_per_minute: Optional[int] | Omit = omit,
        prequeue_start_time: Optional[str] | Omit = omit,
        queueing_method: Optional[str] | Omit = omit,
        session_duration: Optional[int] | Omit = omit,
        shuffle_at_event_start: bool | Omit = omit,
        suspended: bool | Omit = omit,
        total_active_users: Optional[int] | Omit = omit,
        turnstile_action: Optional[Literal["log", "infinite_queue"]] | Omit = omit,
        turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """Only available for the Waiting Room Advanced subscription.

        Creates an event for
        a waiting room. An event takes place during a specified period of time,
        temporarily changing the behavior of a waiting room. While the event is active,
        some of the properties in the event's configuration may either override or
        inherit from the waiting room's configuration. Note that events cannot overlap
        with each other, so only one event can be active at a time.

        Args:
          zone_id: Identifier.

          event_end_time: An ISO 8601 timestamp that marks the end of the event.

          event_start_time: An ISO 8601 timestamp that marks the start of the event. At this time, queued
              users will be processed with the event's configuration. The start time must be
              at least one minute before `event_end_time`.

          name: A unique name to identify the event. Only alphanumeric characters, hyphens and
              underscores are allowed.

          custom_page_html: If set, the event will override the waiting room's `custom_page_html` property
              while it is active. If null, the event will inherit it.

          description: A note that you can use to add more details about the event.

          disable_session_renewal: If set, the event will override the waiting room's `disable_session_renewal`
              property while it is active. If null, the event will inherit it.

          new_users_per_minute: If set, the event will override the waiting room's `new_users_per_minute`
              property while it is active. If null, the event will inherit it. This can only
              be set if the event's `total_active_users` property is also set.

          prequeue_start_time: An ISO 8601 timestamp that marks when to begin queueing all users before the
              event starts. The prequeue must start at least five minutes before
              `event_start_time`.

          queueing_method: If set, the event will override the waiting room's `queueing_method` property
              while it is active. If null, the event will inherit it.

          session_duration: If set, the event will override the waiting room's `session_duration` property
              while it is active. If null, the event will inherit it.

          shuffle_at_event_start: If enabled, users in the prequeue will be shuffled randomly at the
              `event_start_time`. Requires that `prequeue_start_time` is not null. This is
              useful for situations when many users will join the event prequeue at the same
              time and you want to shuffle them to ensure fairness. Naturally, it makes the
              most sense to enable this feature when the `queueing_method` during the event
              respects ordering such as **fifo**, or else the shuffling may be unnecessary.

          suspended: Suspends or allows an event. If set to `true`, the event is ignored and traffic
              will be handled based on the waiting room configuration.

          total_active_users: If set, the event will override the waiting room's `total_active_users` property
              while it is active. If null, the event will inherit it. This can only be set if
              the event's `new_users_per_minute` property is also set.

          turnstile_action: If set, the event will override the waiting room's `turnstile_action` property
              while it is active. If null, the event will inherit it.

          turnstile_mode: If set, the event will override the waiting room's `turnstile_mode` property
              while it is active. If null, the event will inherit it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        return self._post(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
            ),
            body=maybe_transform(
                {
                    "event_end_time": event_end_time,
                    "event_start_time": event_start_time,
                    "name": name,
                    "custom_page_html": custom_page_html,
                    "description": description,
                    "disable_session_renewal": disable_session_renewal,
                    "new_users_per_minute": new_users_per_minute,
                    "prequeue_start_time": prequeue_start_time,
                    "queueing_method": queueing_method,
                    "session_duration": session_duration,
                    "shuffle_at_event_start": shuffle_at_event_start,
                    "suspended": suspended,
                    "total_active_users": total_active_users,
                    "turnstile_action": turnstile_action,
                    "turnstile_mode": turnstile_mode,
                },
                event_create_params.EventCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )

    def update(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        event_end_time: str,
        event_start_time: str,
        name: str,
        custom_page_html: Optional[str] | Omit = omit,
        description: str | Omit = omit,
        disable_session_renewal: Optional[bool] | Omit = omit,
        new_users_per_minute: Optional[int] | Omit = omit,
        prequeue_start_time: Optional[str] | Omit = omit,
        queueing_method: Optional[str] | Omit = omit,
        session_duration: Optional[int] | Omit = omit,
        shuffle_at_event_start: bool | Omit = omit,
        suspended: bool | Omit = omit,
        total_active_users: Optional[int] | Omit = omit,
        turnstile_action: Optional[Literal["log", "infinite_queue"]] | Omit = omit,
        turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """
        Updates a configured event for a waiting room.

        Args:
          zone_id: Identifier.

          event_end_time: An ISO 8601 timestamp that marks the end of the event.

          event_start_time: An ISO 8601 timestamp that marks the start of the event. At this time, queued
              users will be processed with the event's configuration. The start time must be
              at least one minute before `event_end_time`.

          name: A unique name to identify the event. Only alphanumeric characters, hyphens and
              underscores are allowed.

          custom_page_html: If set, the event will override the waiting room's `custom_page_html` property
              while it is active. If null, the event will inherit it.

          description: A note that you can use to add more details about the event.

          disable_session_renewal: If set, the event will override the waiting room's `disable_session_renewal`
              property while it is active. If null, the event will inherit it.

          new_users_per_minute: If set, the event will override the waiting room's `new_users_per_minute`
              property while it is active. If null, the event will inherit it. This can only
              be set if the event's `total_active_users` property is also set.

          prequeue_start_time: An ISO 8601 timestamp that marks when to begin queueing all users before the
              event starts. The prequeue must start at least five minutes before
              `event_start_time`.

          queueing_method: If set, the event will override the waiting room's `queueing_method` property
              while it is active. If null, the event will inherit it.

          session_duration: If set, the event will override the waiting room's `session_duration` property
              while it is active. If null, the event will inherit it.

          shuffle_at_event_start: If enabled, users in the prequeue will be shuffled randomly at the
              `event_start_time`. Requires that `prequeue_start_time` is not null. This is
              useful for situations when many users will join the event prequeue at the same
              time and you want to shuffle them to ensure fairness. Naturally, it makes the
              most sense to enable this feature when the `queueing_method` during the event
              respects ordering such as **fifo**, or else the shuffling may be unnecessary.

          suspended: Suspends or allows an event. If set to `true`, the event is ignored and traffic
              will be handled based on the waiting room configuration.

          total_active_users: If set, the event will override the waiting room's `total_active_users` property
              while it is active. If null, the event will inherit it. This can only be set if
              the event's `new_users_per_minute` property is also set.

          turnstile_action: If set, the event will override the waiting room's `turnstile_action` property
              while it is active. If null, the event will inherit it.

          turnstile_mode: If set, the event will override the waiting room's `turnstile_mode` property
              while it is active. If null, the event will inherit it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            body=maybe_transform(
                {
                    "event_end_time": event_end_time,
                    "event_start_time": event_start_time,
                    "name": name,
                    "custom_page_html": custom_page_html,
                    "description": description,
                    "disable_session_renewal": disable_session_renewal,
                    "new_users_per_minute": new_users_per_minute,
                    "prequeue_start_time": prequeue_start_time,
                    "queueing_method": queueing_method,
                    "session_duration": session_duration,
                    "shuffle_at_event_start": shuffle_at_event_start,
                    "suspended": suspended,
                    "total_active_users": total_active_users,
                    "turnstile_action": turnstile_action,
                    "turnstile_mode": turnstile_mode,
                },
                event_update_params.EventUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )

    def list(
        self,
        waiting_room_id: str,
        *,
        zone_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Event]:
        """
        Lists events for a waiting room.

        Args:
          zone_id: Identifier.

          page: Page number of paginated results.

          per_page: Maximum number of results per page. Must be a multiple of 5.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        return self._get_api_list(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
            ),
            page=SyncV4PagePaginationArray[Event],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=Event,
        )

    def delete(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventDeleteResponse:
        """
        Deletes an event for a waiting room.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EventDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[EventDeleteResponse], ResultWrapper[EventDeleteResponse]),
        )

    def edit(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        event_end_time: str,
        event_start_time: str,
        name: str,
        custom_page_html: Optional[str] | Omit = omit,
        description: str | Omit = omit,
        disable_session_renewal: Optional[bool] | Omit = omit,
        new_users_per_minute: Optional[int] | Omit = omit,
        prequeue_start_time: Optional[str] | Omit = omit,
        queueing_method: Optional[str] | Omit = omit,
        session_duration: Optional[int] | Omit = omit,
        shuffle_at_event_start: bool | Omit = omit,
        suspended: bool | Omit = omit,
        total_active_users: Optional[int] | Omit = omit,
        turnstile_action: Optional[Literal["log", "infinite_queue"]] | Omit = omit,
        turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """
        Patches a configured event for a waiting room.

        Args:
          zone_id: Identifier.

          event_end_time: An ISO 8601 timestamp that marks the end of the event.

          event_start_time: An ISO 8601 timestamp that marks the start of the event. At this time, queued
              users will be processed with the event's configuration. The start time must be
              at least one minute before `event_end_time`.

          name: A unique name to identify the event. Only alphanumeric characters, hyphens and
              underscores are allowed.

          custom_page_html: If set, the event will override the waiting room's `custom_page_html` property
              while it is active. If null, the event will inherit it.

          description: A note that you can use to add more details about the event.

          disable_session_renewal: If set, the event will override the waiting room's `disable_session_renewal`
              property while it is active. If null, the event will inherit it.

          new_users_per_minute: If set, the event will override the waiting room's `new_users_per_minute`
              property while it is active. If null, the event will inherit it. This can only
              be set if the event's `total_active_users` property is also set.

          prequeue_start_time: An ISO 8601 timestamp that marks when to begin queueing all users before the
              event starts. The prequeue must start at least five minutes before
              `event_start_time`.

          queueing_method: If set, the event will override the waiting room's `queueing_method` property
              while it is active. If null, the event will inherit it.

          session_duration: If set, the event will override the waiting room's `session_duration` property
              while it is active. If null, the event will inherit it.

          shuffle_at_event_start: If enabled, users in the prequeue will be shuffled randomly at the
              `event_start_time`. Requires that `prequeue_start_time` is not null. This is
              useful for situations when many users will join the event prequeue at the same
              time and you want to shuffle them to ensure fairness. Naturally, it makes the
              most sense to enable this feature when the `queueing_method` during the event
              respects ordering such as **fifo**, or else the shuffling may be unnecessary.

          suspended: Suspends or allows an event. If set to `true`, the event is ignored and traffic
              will be handled based on the waiting room configuration.

          total_active_users: If set, the event will override the waiting room's `total_active_users` property
              while it is active. If null, the event will inherit it. This can only be set if
              the event's `new_users_per_minute` property is also set.

          turnstile_action: If set, the event will override the waiting room's `turnstile_action` property
              while it is active. If null, the event will inherit it.

          turnstile_mode: If set, the event will override the waiting room's `turnstile_mode` property
              while it is active. If null, the event will inherit it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._patch(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            body=maybe_transform(
                {
                    "event_end_time": event_end_time,
                    "event_start_time": event_start_time,
                    "name": name,
                    "custom_page_html": custom_page_html,
                    "description": description,
                    "disable_session_renewal": disable_session_renewal,
                    "new_users_per_minute": new_users_per_minute,
                    "prequeue_start_time": prequeue_start_time,
                    "queueing_method": queueing_method,
                    "session_duration": session_duration,
                    "shuffle_at_event_start": shuffle_at_event_start,
                    "suspended": suspended,
                    "total_active_users": total_active_users,
                    "turnstile_action": turnstile_action,
                    "turnstile_mode": turnstile_mode,
                },
                event_edit_params.EventEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )

    def get(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """
        Fetches a single configured event for a waiting room.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def details(self) -> AsyncDetailsResource:
        return AsyncDetailsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    async def create(
        self,
        waiting_room_id: str,
        *,
        zone_id: str,
        event_end_time: str,
        event_start_time: str,
        name: str,
        custom_page_html: Optional[str] | Omit = omit,
        description: str | Omit = omit,
        disable_session_renewal: Optional[bool] | Omit = omit,
        new_users_per_minute: Optional[int] | Omit = omit,
        prequeue_start_time: Optional[str] | Omit = omit,
        queueing_method: Optional[str] | Omit = omit,
        session_duration: Optional[int] | Omit = omit,
        shuffle_at_event_start: bool | Omit = omit,
        suspended: bool | Omit = omit,
        total_active_users: Optional[int] | Omit = omit,
        turnstile_action: Optional[Literal["log", "infinite_queue"]] | Omit = omit,
        turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """Only available for the Waiting Room Advanced subscription.

        Creates an event for
        a waiting room. An event takes place during a specified period of time,
        temporarily changing the behavior of a waiting room. While the event is active,
        some of the properties in the event's configuration may either override or
        inherit from the waiting room's configuration. Note that events cannot overlap
        with each other, so only one event can be active at a time.

        Args:
          zone_id: Identifier.

          event_end_time: An ISO 8601 timestamp that marks the end of the event.

          event_start_time: An ISO 8601 timestamp that marks the start of the event. At this time, queued
              users will be processed with the event's configuration. The start time must be
              at least one minute before `event_end_time`.

          name: A unique name to identify the event. Only alphanumeric characters, hyphens and
              underscores are allowed.

          custom_page_html: If set, the event will override the waiting room's `custom_page_html` property
              while it is active. If null, the event will inherit it.

          description: A note that you can use to add more details about the event.

          disable_session_renewal: If set, the event will override the waiting room's `disable_session_renewal`
              property while it is active. If null, the event will inherit it.

          new_users_per_minute: If set, the event will override the waiting room's `new_users_per_minute`
              property while it is active. If null, the event will inherit it. This can only
              be set if the event's `total_active_users` property is also set.

          prequeue_start_time: An ISO 8601 timestamp that marks when to begin queueing all users before the
              event starts. The prequeue must start at least five minutes before
              `event_start_time`.

          queueing_method: If set, the event will override the waiting room's `queueing_method` property
              while it is active. If null, the event will inherit it.

          session_duration: If set, the event will override the waiting room's `session_duration` property
              while it is active. If null, the event will inherit it.

          shuffle_at_event_start: If enabled, users in the prequeue will be shuffled randomly at the
              `event_start_time`. Requires that `prequeue_start_time` is not null. This is
              useful for situations when many users will join the event prequeue at the same
              time and you want to shuffle them to ensure fairness. Naturally, it makes the
              most sense to enable this feature when the `queueing_method` during the event
              respects ordering such as **fifo**, or else the shuffling may be unnecessary.

          suspended: Suspends or allows an event. If set to `true`, the event is ignored and traffic
              will be handled based on the waiting room configuration.

          total_active_users: If set, the event will override the waiting room's `total_active_users` property
              while it is active. If null, the event will inherit it. This can only be set if
              the event's `new_users_per_minute` property is also set.

          turnstile_action: If set, the event will override the waiting room's `turnstile_action` property
              while it is active. If null, the event will inherit it.

          turnstile_mode: If set, the event will override the waiting room's `turnstile_mode` property
              while it is active. If null, the event will inherit it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        return await self._post(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
            ),
            body=await async_maybe_transform(
                {
                    "event_end_time": event_end_time,
                    "event_start_time": event_start_time,
                    "name": name,
                    "custom_page_html": custom_page_html,
                    "description": description,
                    "disable_session_renewal": disable_session_renewal,
                    "new_users_per_minute": new_users_per_minute,
                    "prequeue_start_time": prequeue_start_time,
                    "queueing_method": queueing_method,
                    "session_duration": session_duration,
                    "shuffle_at_event_start": shuffle_at_event_start,
                    "suspended": suspended,
                    "total_active_users": total_active_users,
                    "turnstile_action": turnstile_action,
                    "turnstile_mode": turnstile_mode,
                },
                event_create_params.EventCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )

    async def update(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        event_end_time: str,
        event_start_time: str,
        name: str,
        custom_page_html: Optional[str] | Omit = omit,
        description: str | Omit = omit,
        disable_session_renewal: Optional[bool] | Omit = omit,
        new_users_per_minute: Optional[int] | Omit = omit,
        prequeue_start_time: Optional[str] | Omit = omit,
        queueing_method: Optional[str] | Omit = omit,
        session_duration: Optional[int] | Omit = omit,
        shuffle_at_event_start: bool | Omit = omit,
        suspended: bool | Omit = omit,
        total_active_users: Optional[int] | Omit = omit,
        turnstile_action: Optional[Literal["log", "infinite_queue"]] | Omit = omit,
        turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """
        Updates a configured event for a waiting room.

        Args:
          zone_id: Identifier.

          event_end_time: An ISO 8601 timestamp that marks the end of the event.

          event_start_time: An ISO 8601 timestamp that marks the start of the event. At this time, queued
              users will be processed with the event's configuration. The start time must be
              at least one minute before `event_end_time`.

          name: A unique name to identify the event. Only alphanumeric characters, hyphens and
              underscores are allowed.

          custom_page_html: If set, the event will override the waiting room's `custom_page_html` property
              while it is active. If null, the event will inherit it.

          description: A note that you can use to add more details about the event.

          disable_session_renewal: If set, the event will override the waiting room's `disable_session_renewal`
              property while it is active. If null, the event will inherit it.

          new_users_per_minute: If set, the event will override the waiting room's `new_users_per_minute`
              property while it is active. If null, the event will inherit it. This can only
              be set if the event's `total_active_users` property is also set.

          prequeue_start_time: An ISO 8601 timestamp that marks when to begin queueing all users before the
              event starts. The prequeue must start at least five minutes before
              `event_start_time`.

          queueing_method: If set, the event will override the waiting room's `queueing_method` property
              while it is active. If null, the event will inherit it.

          session_duration: If set, the event will override the waiting room's `session_duration` property
              while it is active. If null, the event will inherit it.

          shuffle_at_event_start: If enabled, users in the prequeue will be shuffled randomly at the
              `event_start_time`. Requires that `prequeue_start_time` is not null. This is
              useful for situations when many users will join the event prequeue at the same
              time and you want to shuffle them to ensure fairness. Naturally, it makes the
              most sense to enable this feature when the `queueing_method` during the event
              respects ordering such as **fifo**, or else the shuffling may be unnecessary.

          suspended: Suspends or allows an event. If set to `true`, the event is ignored and traffic
              will be handled based on the waiting room configuration.

          total_active_users: If set, the event will override the waiting room's `total_active_users` property
              while it is active. If null, the event will inherit it. This can only be set if
              the event's `new_users_per_minute` property is also set.

          turnstile_action: If set, the event will override the waiting room's `turnstile_action` property
              while it is active. If null, the event will inherit it.

          turnstile_mode: If set, the event will override the waiting room's `turnstile_mode` property
              while it is active. If null, the event will inherit it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            body=await async_maybe_transform(
                {
                    "event_end_time": event_end_time,
                    "event_start_time": event_start_time,
                    "name": name,
                    "custom_page_html": custom_page_html,
                    "description": description,
                    "disable_session_renewal": disable_session_renewal,
                    "new_users_per_minute": new_users_per_minute,
                    "prequeue_start_time": prequeue_start_time,
                    "queueing_method": queueing_method,
                    "session_duration": session_duration,
                    "shuffle_at_event_start": shuffle_at_event_start,
                    "suspended": suspended,
                    "total_active_users": total_active_users,
                    "turnstile_action": turnstile_action,
                    "turnstile_mode": turnstile_mode,
                },
                event_update_params.EventUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )

    def list(
        self,
        waiting_room_id: str,
        *,
        zone_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Event, AsyncV4PagePaginationArray[Event]]:
        """
        Lists events for a waiting room.

        Args:
          zone_id: Identifier.

          page: Page number of paginated results.

          per_page: Maximum number of results per page. Must be a multiple of 5.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        return self._get_api_list(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
            ),
            page=AsyncV4PagePaginationArray[Event],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=Event,
        )

    async def delete(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventDeleteResponse:
        """
        Deletes an event for a waiting room.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EventDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[EventDeleteResponse], ResultWrapper[EventDeleteResponse]),
        )

    async def edit(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        event_end_time: str,
        event_start_time: str,
        name: str,
        custom_page_html: Optional[str] | Omit = omit,
        description: str | Omit = omit,
        disable_session_renewal: Optional[bool] | Omit = omit,
        new_users_per_minute: Optional[int] | Omit = omit,
        prequeue_start_time: Optional[str] | Omit = omit,
        queueing_method: Optional[str] | Omit = omit,
        session_duration: Optional[int] | Omit = omit,
        shuffle_at_event_start: bool | Omit = omit,
        suspended: bool | Omit = omit,
        total_active_users: Optional[int] | Omit = omit,
        turnstile_action: Optional[Literal["log", "infinite_queue"]] | Omit = omit,
        turnstile_mode: Optional[Literal["off", "invisible", "visible_non_interactive", "visible_managed"]]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """
        Patches a configured event for a waiting room.

        Args:
          zone_id: Identifier.

          event_end_time: An ISO 8601 timestamp that marks the end of the event.

          event_start_time: An ISO 8601 timestamp that marks the start of the event. At this time, queued
              users will be processed with the event's configuration. The start time must be
              at least one minute before `event_end_time`.

          name: A unique name to identify the event. Only alphanumeric characters, hyphens and
              underscores are allowed.

          custom_page_html: If set, the event will override the waiting room's `custom_page_html` property
              while it is active. If null, the event will inherit it.

          description: A note that you can use to add more details about the event.

          disable_session_renewal: If set, the event will override the waiting room's `disable_session_renewal`
              property while it is active. If null, the event will inherit it.

          new_users_per_minute: If set, the event will override the waiting room's `new_users_per_minute`
              property while it is active. If null, the event will inherit it. This can only
              be set if the event's `total_active_users` property is also set.

          prequeue_start_time: An ISO 8601 timestamp that marks when to begin queueing all users before the
              event starts. The prequeue must start at least five minutes before
              `event_start_time`.

          queueing_method: If set, the event will override the waiting room's `queueing_method` property
              while it is active. If null, the event will inherit it.

          session_duration: If set, the event will override the waiting room's `session_duration` property
              while it is active. If null, the event will inherit it.

          shuffle_at_event_start: If enabled, users in the prequeue will be shuffled randomly at the
              `event_start_time`. Requires that `prequeue_start_time` is not null. This is
              useful for situations when many users will join the event prequeue at the same
              time and you want to shuffle them to ensure fairness. Naturally, it makes the
              most sense to enable this feature when the `queueing_method` during the event
              respects ordering such as **fifo**, or else the shuffling may be unnecessary.

          suspended: Suspends or allows an event. If set to `true`, the event is ignored and traffic
              will be handled based on the waiting room configuration.

          total_active_users: If set, the event will override the waiting room's `total_active_users` property
              while it is active. If null, the event will inherit it. This can only be set if
              the event's `new_users_per_minute` property is also set.

          turnstile_action: If set, the event will override the waiting room's `turnstile_action` property
              while it is active. If null, the event will inherit it.

          turnstile_mode: If set, the event will override the waiting room's `turnstile_mode` property
              while it is active. If null, the event will inherit it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._patch(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            body=await async_maybe_transform(
                {
                    "event_end_time": event_end_time,
                    "event_start_time": event_start_time,
                    "name": name,
                    "custom_page_html": custom_page_html,
                    "description": description,
                    "disable_session_renewal": disable_session_renewal,
                    "new_users_per_minute": new_users_per_minute,
                    "prequeue_start_time": prequeue_start_time,
                    "queueing_method": queueing_method,
                    "session_duration": session_duration,
                    "shuffle_at_event_start": shuffle_at_event_start,
                    "suspended": suspended,
                    "total_active_users": total_active_users,
                    "turnstile_action": turnstile_action,
                    "turnstile_mode": turnstile_mode,
                },
                event_edit_params.EventEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )

    async def get(
        self,
        event_id: str,
        *,
        zone_id: str,
        waiting_room_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """
        Fetches a single configured event for a waiting room.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/events/{event_id}",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Event]._unwrapper,
            ),
            cast_to=cast(Type[Event], ResultWrapper[Event]),
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.create = to_raw_response_wrapper(
            events.create,
        )
        self.update = to_raw_response_wrapper(
            events.update,
        )
        self.list = to_raw_response_wrapper(
            events.list,
        )
        self.delete = to_raw_response_wrapper(
            events.delete,
        )
        self.edit = to_raw_response_wrapper(
            events.edit,
        )
        self.get = to_raw_response_wrapper(
            events.get,
        )

    @cached_property
    def details(self) -> DetailsResourceWithRawResponse:
        return DetailsResourceWithRawResponse(self._events.details)


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.create = async_to_raw_response_wrapper(
            events.create,
        )
        self.update = async_to_raw_response_wrapper(
            events.update,
        )
        self.list = async_to_raw_response_wrapper(
            events.list,
        )
        self.delete = async_to_raw_response_wrapper(
            events.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            events.edit,
        )
        self.get = async_to_raw_response_wrapper(
            events.get,
        )

    @cached_property
    def details(self) -> AsyncDetailsResourceWithRawResponse:
        return AsyncDetailsResourceWithRawResponse(self._events.details)


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.create = to_streamed_response_wrapper(
            events.create,
        )
        self.update = to_streamed_response_wrapper(
            events.update,
        )
        self.list = to_streamed_response_wrapper(
            events.list,
        )
        self.delete = to_streamed_response_wrapper(
            events.delete,
        )
        self.edit = to_streamed_response_wrapper(
            events.edit,
        )
        self.get = to_streamed_response_wrapper(
            events.get,
        )

    @cached_property
    def details(self) -> DetailsResourceWithStreamingResponse:
        return DetailsResourceWithStreamingResponse(self._events.details)


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.create = async_to_streamed_response_wrapper(
            events.create,
        )
        self.update = async_to_streamed_response_wrapper(
            events.update,
        )
        self.list = async_to_streamed_response_wrapper(
            events.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            events.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            events.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            events.get,
        )

    @cached_property
    def details(self) -> AsyncDetailsResourceWithStreamingResponse:
        return AsyncDetailsResourceWithStreamingResponse(self._events.details)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from .ipv6_network import IPV6Network

__all__ = ["IPV6Endpoint"]


class IPV6Endpoint(BaseModel):
    enabled: Optional[bool] = None
    """Indicate whether the IPV6 endpoint is enabled for this location."""

    networks: Optional[List[IPV6Network]] = None
    """Specify the list of allowed source IPv6 network ranges for this endpoint.

    When the list is empty, the endpoint allows all source IPs. The list takes
    effect only if the endpoint is enabled for this location.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MatcherParam"]


class MatcherParam(TypedDict, total=False):
    """Matching pattern to forward your actions."""

    type: Required[Literal["all", "literal"]]
    """Type of matcher."""

    field: Literal["to"]
    """Field for type matcher."""

    value: str
    """Value for matcher."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict

from ..._models import BaseModel

__all__ = ["ReverseDNSGetResponse"]


class ReverseDNSGetResponse(BaseModel):
    ptr: Dict[str, str]
    """Map of cluster IP addresses to PTR record contents"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ....._compat import cached_property
from .rules.rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource
from .filters.filters import (
    FiltersResource,
    AsyncFiltersResource,
    FiltersResourceWithRawResponse,
    AsyncFiltersResourceWithRawResponse,
    FiltersResourceWithStreamingResponse,
    AsyncFiltersResourceWithStreamingResponse,
)

__all__ = ["SynProtectionResource", "AsyncSynProtectionResource"]


class SynProtectionResource(SyncAPIResource):
    @cached_property
    def filters(self) -> FiltersResource:
        return FiltersResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> SynProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SynProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SynProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SynProtectionResourceWithStreamingResponse(self)


class AsyncSynProtectionResource(AsyncAPIResource):
    @cached_property
    def filters(self) -> AsyncFiltersResource:
        return AsyncFiltersResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSynProtectionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSynProtectionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSynProtectionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSynProtectionResourceWithStreamingResponse(self)


class SynProtectionResourceWithRawResponse:
    def __init__(self, syn_protection: SynProtectionResource) -> None:
        self._syn_protection = syn_protection

    @cached_property
    def filters(self) -> FiltersResourceWithRawResponse:
        return FiltersResourceWithRawResponse(self._syn_protection.filters)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._syn_protection.rules)


class AsyncSynProtectionResourceWithRawResponse:
    def __init__(self, syn_protection: AsyncSynProtectionResource) -> None:
        self._syn_protection = syn_protection

    @cached_property
    def filters(self) -> AsyncFiltersResourceWithRawResponse:
        return AsyncFiltersResourceWithRawResponse(self._syn_protection.filters)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._syn_protection.rules)


class SynProtectionResourceWithStreamingResponse:
    def __init__(self, syn_protection: SynProtectionResource) -> None:
        self._syn_protection = syn_protection

    @cached_property
    def filters(self) -> FiltersResourceWithStreamingResponse:
        return FiltersResourceWithStreamingResponse(self._syn_protection.filters)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._syn_protection.rules)


class AsyncSynProtectionResourceWithStreamingResponse:
    def __init__(self, syn_protection: AsyncSynProtectionResource) -> None:
        self._syn_protection = syn_protection

    @cached_property
    def filters(self) -> AsyncFiltersResourceWithStreamingResponse:
        return AsyncFiltersResourceWithStreamingResponse(self._syn_protection.filters)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._syn_protection.rules)
